from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.math import *
from OCC.Core.Adaptor3d import *
from OCC.Core.IntImp import *
from OCC.Core.TColStd import *
from OCC.Core.gp import *
from OCC.Core.IntSurf import *


class IntWalk_StatusDeflection(IntEnum):
	IntWalk_PasTropGrand: int = ...
	IntWalk_StepTooSmall: int = ...
	IntWalk_PointConfondu: int = ...
	IntWalk_ArretSurPointPrecedent: int = ...
	IntWalk_ArretSurPoint: int = ...
	IntWalk_OK: int = ...
IntWalk_PasTropGrand = IntWalk_StatusDeflection.IntWalk_PasTropGrand
IntWalk_StepTooSmall = IntWalk_StatusDeflection.IntWalk_StepTooSmall
IntWalk_PointConfondu = IntWalk_StatusDeflection.IntWalk_PointConfondu
IntWalk_ArretSurPointPrecedent = IntWalk_StatusDeflection.IntWalk_ArretSurPointPrecedent
IntWalk_ArretSurPoint = IntWalk_StatusDeflection.IntWalk_ArretSurPoint
IntWalk_OK = IntWalk_StatusDeflection.IntWalk_OK

class IntWalk_TheFunctionOfTheInt2S(math_FunctionSetWithDerivatives):
	def __init__(self, S1: Adaptor3d_HSurface, S2: Adaptor3d_HSurface) -> None: ...
	def AuxillarSurface1(self) -> Adaptor3d_HSurface: ...
	def AuxillarSurface2(self) -> Adaptor3d_HSurface: ...
	def ComputeParameters(self, ChoixIso: IntImp_ConstIsoparametric, Param: TColStd_Array1OfReal, UVap: math_Vector, BornInf: math_Vector, BornSup: math_Vector, Tolerance: math_Vector) -> None: ...
	def Derivatives(self, X: math_Vector, D: math_Matrix) -> bool: ...
	def Direction(self) -> gp_Dir: ...
	def DirectionOnS1(self) -> gp_Dir2d: ...
	def DirectionOnS2(self) -> gp_Dir2d: ...
	def IsTangent(self, UVap: math_Vector, Param: TColStd_Array1OfReal, BestChoix: IntImp_ConstIsoparametric) -> bool: ...
	def NbEquations(self) -> int: ...
	def NbVariables(self) -> int: ...
	def Point(self) -> gp_Pnt: ...
	def Root(self) -> float: ...
	def Value(self, X: math_Vector, F: math_Vector) -> bool: ...
	def Values(self, X: math_Vector, F: math_Vector, D: math_Matrix) -> bool: ...

class IntWalk_TheInt2S:
	@overload
	def __init__(self, Param: TColStd_Array1OfReal, S1: Adaptor3d_HSurface, S2: Adaptor3d_HSurface, TolTangency: float) -> None: ...
	@overload
	def __init__(self, S1: Adaptor3d_HSurface, S2: Adaptor3d_HSurface, TolTangency: float) -> None: ...
	def ChangePoint(self) -> IntSurf_PntOn2S: ...
	def Direction(self) -> gp_Dir: ...
	def DirectionOnS1(self) -> gp_Dir2d: ...
	def DirectionOnS2(self) -> gp_Dir2d: ...
	def Function(self) -> IntWalk_TheFunctionOfTheInt2S: ...
	def IsDone(self) -> bool: ...
	def IsEmpty(self) -> bool: ...
	def IsTangent(self) -> bool: ...
	@overload
	def Perform(self, Param: TColStd_Array1OfReal, Rsnld: math_FunctionSetRoot) -> IntImp_ConstIsoparametric: ...
	@overload
	def Perform(self, Param: TColStd_Array1OfReal, Rsnld: math_FunctionSetRoot, ChoixIso: IntImp_ConstIsoparametric) -> IntImp_ConstIsoparametric: ...
	def Point(self) -> IntSurf_PntOn2S: ...

class IntWalk_WalkingData:
	pass

# harray1 classes
# harray2 classes
# hsequence classes




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
StepAP209 module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_stepap209.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _StepAP209
else:
    import _StepAP209

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _StepAP209.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _StepAP209.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _StepAP209.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _StepAP209.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _StepAP209.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _StepAP209.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _StepAP209.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _StepAP209.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _StepAP209.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _StepAP209.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _StepAP209.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _StepAP209.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _StepAP209.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _StepAP209.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _StepAP209.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _StepAP209.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _StepAP209.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _StepAP209:
_StepAP209.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _StepAP209.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.STEPConstruct
import OCC.Core.Transfer
import OCC.Core.Message
import OCC.Core.TCollection
import OCC.Core.OSD
import OCC.Core.TColStd
import OCC.Core.Interface
import OCC.Core.MoniTool
import OCC.Core.TopoDS
import OCC.Core.TopAbs
import OCC.Core.TopLoc
import OCC.Core.gp
import OCC.Core.StepShape
import OCC.Core.StepGeom
import OCC.Core.StepData
import OCC.Core.Resource
import OCC.Core.StepRepr
import OCC.Core.StepBasic
import OCC.Core.StepAP203
import OCC.Core.XSControl
import OCC.Core.IFSelect
import OCC.Core.TopTools
import OCC.Core.Geom
import OCC.Core.GeomAbs
import OCC.Core.TColgp
import OCC.Core.Geom2d
import OCC.Core.StepVisual
import OCC.Core.Quantity
import OCC.Core.StepFEA
import OCC.Core.StepElement

from enum import IntEnum
from OCC.Core.Exception import *



class StepAP209_Construct(OCC.Core.STEPConstruct.STEPConstruct_Tool):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")
    __repr__ = _swig_repr

    def __init__(self, *args):
        r"""
        Creates an empty tool.

        Returns
        -------
        None
        Creates a tool and initializes it.

        Parameters
        ----------
        WS: XSControl_WorkSession

        Returns
        -------
        None

        """
        _StepAP209.StepAP209_Construct_swiginit(self, _StepAP209.new_StepAP209_Construct(*args))

    def CreateAP203Structure(self, *args) -> "opencascade::handle< StepData_StepModel >":
        r"""
        Create ap203 structure from existing ap209 structure.

        Returns
        -------
        opencascade::handle<StepData_StepModel>

        """
        return _StepAP209.StepAP209_Construct_CreateAP203Structure(self, *args)

    def CreateAdding203Entities(self, *args) -> "Standard_Boolean":
        r"""
        Create approval.. , date.. , time.. , person.. and organization.. entities for 203 structure.

        Parameters
        ----------
        PD: StepBasic_ProductDefinition
        aModel: StepData_StepModel

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_CreateAdding203Entities(self, *args)

    def CreateAddingEntities(self, *args) -> "Standard_Boolean":
        r"""
        Create approval.. , date.. , time.. , person.. and organization.. entities for analysis structure.

        Parameters
        ----------
        AnaPD: StepBasic_ProductDefinition

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_CreateAddingEntities(self, *args)

    def CreateAnalysStructure(self, *args) -> "Standard_Boolean":
        r"""
        Create empty structure for idealized_analysis_shape.

        Parameters
        ----------
        Prod: StepBasic_Product

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_CreateAnalysStructure(self, *args)

    def CreateFeaStructure(self, *args) -> "Standard_Boolean":
        r"""
        Create fea structure.

        Parameters
        ----------
        Prod: StepBasic_Product

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_CreateFeaStructure(self, *args)

    def FeaModel(self, *args) -> "opencascade::handle< StepFEA_FeaModel >":
        r"""
        No available documentation.

        Parameters
        ----------
        Prod: StepBasic_Product

        Returns
        -------
        opencascade::handle<StepFEA_FeaModel>
        No available documentation.

        Parameters
        ----------
        PDF: StepBasic_ProductDefinitionFormation

        Returns
        -------
        opencascade::handle<StepFEA_FeaModel>
        No available documentation.

        Parameters
        ----------
        PDS: StepRepr_ProductDefinitionShape

        Returns
        -------
        opencascade::handle<StepFEA_FeaModel>
        No available documentation.

        Parameters
        ----------
        PD: StepBasic_ProductDefinition

        Returns
        -------
        opencascade::handle<StepFEA_FeaModel>

        """
        return _StepAP209.StepAP209_Construct_FeaModel(self, *args)

    def GetCurElemSection(self, *args) -> "opencascade::handle< StepElement_HSequenceOfCurveElementSectionDefinition >":
        r"""
        Getting list of curve_element_section_definitions for given element_representation.

        Parameters
        ----------
        ElemRepr: StepFEA_Curve3dElementRepresentation

        Returns
        -------
        opencascade::handle<StepElement_HSequenceOfCurveElementSectionDefinition>

        """
        return _StepAP209.StepAP209_Construct_GetCurElemSection(self, *args)

    def GetElemGeomRelat(self, *args) -> "opencascade::handle< StepFEA_HSequenceOfElementGeometricRelationship >":
        r"""
        No available documentation.

        Returns
        -------
        opencascade::handle<StepFEA_HSequenceOfElementGeometricRelationship>

        """
        return _StepAP209.StepAP209_Construct_GetElemGeomRelat(self, *args)

    def GetElementMaterial(self, *args) -> "opencascade::handle< StepElement_HSequenceOfElementMaterial >":
        r"""
        No available documentation.

        Returns
        -------
        opencascade::handle<StepElement_HSequenceOfElementMaterial>

        """
        return _StepAP209.StepAP209_Construct_GetElementMaterial(self, *args)

    def GetElements1D(self, *args) -> "opencascade::handle< StepFEA_HSequenceOfElementRepresentation >":
        r"""
        No available documentation.

        Parameters
        ----------
        theFeaModel: StepFEA_FeaModel

        Returns
        -------
        opencascade::handle<StepFEA_HSequenceOfElementRepresentation>

        """
        return _StepAP209.StepAP209_Construct_GetElements1D(self, *args)

    def GetElements2D(self, *args) -> "opencascade::handle< StepFEA_HSequenceOfElementRepresentation >":
        r"""
        No available documentation.

        Parameters
        ----------
        theFEAModel: StepFEA_FeaModel

        Returns
        -------
        opencascade::handle<StepFEA_HSequenceOfElementRepresentation>

        """
        return _StepAP209.StepAP209_Construct_GetElements2D(self, *args)

    def GetElements3D(self, *args) -> "opencascade::handle< StepFEA_HSequenceOfElementRepresentation >":
        r"""
        No available documentation.

        Parameters
        ----------
        theFEAModel: StepFEA_FeaModel

        Returns
        -------
        opencascade::handle<StepFEA_HSequenceOfElementRepresentation>

        """
        return _StepAP209.StepAP209_Construct_GetElements3D(self, *args)

    def GetFeaAxis2Placement3d(self, *args) -> "opencascade::handle< StepFEA_FeaAxis2Placement3d >":
        r"""
        No available documentation.

        Parameters
        ----------
        theFeaModel: StepFEA_FeaModel

        Returns
        -------
        opencascade::handle<StepFEA_FeaAxis2Placement3d>

        """
        return _StepAP209.StepAP209_Construct_GetFeaAxis2Placement3d(self, *args)

    def GetShReprForElem(self, *args) -> "opencascade::handle< StepShape_ShapeRepresentation >":
        r"""
        No available documentation.

        Parameters
        ----------
        ElemRepr: StepFEA_ElementRepresentation

        Returns
        -------
        opencascade::handle<StepShape_ShapeRepresentation>

        """
        return _StepAP209.StepAP209_Construct_GetShReprForElem(self, *args)

    def IdealShape(self, *args) -> "opencascade::handle< StepShape_ShapeRepresentation >":
        r"""
        No available documentation.

        Parameters
        ----------
        Prod: StepBasic_Product

        Returns
        -------
        opencascade::handle<StepShape_ShapeRepresentation>
        No available documentation.

        Parameters
        ----------
        PDF: StepBasic_ProductDefinitionFormation

        Returns
        -------
        opencascade::handle<StepShape_ShapeRepresentation>
        No available documentation.

        Parameters
        ----------
        PD: StepBasic_ProductDefinition

        Returns
        -------
        opencascade::handle<StepShape_ShapeRepresentation>
        No available documentation.

        Parameters
        ----------
        PDS: StepRepr_ProductDefinitionShape

        Returns
        -------
        opencascade::handle<StepShape_ShapeRepresentation>

        """
        return _StepAP209.StepAP209_Construct_IdealShape(self, *args)

    def Init(self, *args) -> "Standard_Boolean":
        r"""
        Initializes tool; returns true if succeeded.

        Parameters
        ----------
        WS: XSControl_WorkSession

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_Init(self, *args)

    def IsAnalys(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        PD: StepBasic_ProductDefinitionFormation

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_IsAnalys(self, *args)

    def IsDesing(self, *args) -> "Standard_Boolean":
        r"""
        No available documentation.

        Parameters
        ----------
        PD: StepBasic_ProductDefinitionFormation

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_IsDesing(self, *args)

    def NominShape(self, *args) -> "opencascade::handle< StepShape_ShapeRepresentation >":
        r"""
        No available documentation.

        Parameters
        ----------
        Prod: StepBasic_Product

        Returns
        -------
        opencascade::handle<StepShape_ShapeRepresentation>
        No available documentation.

        Parameters
        ----------
        PDF: StepBasic_ProductDefinitionFormation

        Returns
        -------
        opencascade::handle<StepShape_ShapeRepresentation>

        """
        return _StepAP209.StepAP209_Construct_NominShape(self, *args)

    def ReplaceCcDesingToApplied(self, *args) -> "Standard_Boolean":
        r"""
        Put into model entities applied... for ap209 instead of entities ccdesing... from ap203.

        Returns
        -------
        bool

        """
        return _StepAP209.StepAP209_Construct_ReplaceCcDesingToApplied(self, *args)

    __repr__ = _dumps_object

    __swig_destroy__ = _StepAP209.delete_StepAP209_Construct

# Register StepAP209_Construct in _StepAP209:
_StepAP209.StepAP209_Construct_swigregister(StepAP209_Construct)





